/* EntryGroupMenu.java
 *
 * created: Fri Aug 27 1999
 *
 * This file is part of Artemis
 *
 * Copyright (C) 1999  Genome Research Limited
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/disk222/yeastpub/Repository/powmap/diana/components/EntryGroupMenu.java,v 1.1 1999/08/27 13:11:05 kmr Exp $
 */

package diana.components;

import diana.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

/**
 *  A menu containing the current entries in an EntryGroup.
 *
 *  @author Kim Rutherford
 *  @version $Id: EntryGroupMenu.java,v 1.1 1999/08/27 13:11:05 kmr Exp $
 **/

public class EntryGroupMenu extends Menu
    implements EntryGroupChangeListener, EntryChangeListener {
  /**
   *  Create a new EntryGroupMenu object that will show the entries in the
   *  given EntryGroup.
   *  @param frame The Frame that owns this Menu.
   *  @param entry_group The EntryGroup object to display.
   **/
  public EntryGroupMenu (final Frame frame,
                         final EntryGroup entry_group) {
    super ("Entries");

    this.frame = frame;
    this.entry_group = entry_group;

    entry_group.addEntryGroupChangeListener (this);
    entry_group.addEntryChangeListener (this);

    refreshButtons ();
  }

  /**
   *  Implementation of the EntryGroupChangeListener interface.  We listen to
   *  EntryGroupChange events so that we can update the display if entries
   *  are added or deleted.
   **/
  public void entryGroupChanged (final EntryGroupChangeEvent event) {
//      System.out.println ("in FeatureList entry group change event type: " +
//                          event.getType () + " for entry " +
//                          event.getEntry ());

    switch (event.getType ()) {
    case EntryGroupChangeEvent.ENTRY_ADDED:
    case EntryGroupChangeEvent.ENTRY_DELETED:
    case EntryGroupChangeEvent.ENTRY_INACTIVE:
    case EntryGroupChangeEvent.ENTRY_ACTIVE:
      refreshButtons ();
      break;
    case EntryGroupChangeEvent.NEW_DEFAULT_ENTRY:
      // we can't display the default entry
      break;
    }
  }

  /**
   *  Implementation of the EntryChangeListener interface.
   **/
  public void entryChanged (final EntryChangeEvent event) {
//     System.out.println ("in EntryGroup: entry change event type: " +
//                         event.getType () + " for entry " +
//                         event.getSource () + " for feature " +
//                         event.getFeature ());

    if (event.getType () == EntryChangeEvent.NAME_CHANGED) {
      refreshButtons ();
    }
  }

  /**
   *  Update the Buttons to the reflect the current contents of the
   *  EntryGroup.
   **/
  private void refreshButtons () {
    removeAll ();

    final MenuItem set_default = new MenuItem ("Set Default Entry ...");
    set_default.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent event) {
        setDefaultEntry ();
      }
    });

    add (set_default);

    final MenuItem set_default_entry_name =
      new MenuItem ("Set Name Of Default Entry ...");
    set_default_entry_name.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent event) {
        setDefaultEntryName ();
      }
    });

    add (set_default_entry_name);

    addSeparator ();

    entry_components = new Vector ();

    if (entry_group == null) {
      return;
    } else {
      for (int i = 0 ; i < entry_group.size () ; ++i) {
        add (entry_group.elementAt (i));
      }
    }
  }

  /**
   *  Add a Label or Checkbox for the given Entry to this component.
   **/
  private void add (final Entry entry) {
    String entry_name = entry.getName ();

    if (entry_name == null) {
      entry_name = "no name";
    }

    final CheckboxMenuItem new_component =
      new CheckboxMenuItem (entry_name, entry_group.isActive (entry));

    new_component.addItemListener (new ItemListener () {
      public void itemStateChanged (ItemEvent event) {
        final int button_index =
          entry_components.indexOf (event.getSource ());

        if (event.getStateChange () == ItemEvent.SELECTED) {
          entry_group.setIsActive (button_index, true);
        } else {
          entry_group.setIsActive (button_index, false);
        }
      }
    });

    entry_components.addElement (new_component);
    add (new_component);
  }

  /**
   *  Create a new TextRequester component and set the name of the default
   *  Entry to whatever the user types.
   **/
  private void setDefaultEntryName () {
    if (entry_group.getDefaultEntry () == null) {
      new MessageDialog (frame, "There is no default entry");
      return;
    }

    final TextRequester text_requester =
      new TextRequester ("New name for the default entry?", 18, "");

    text_requester.addTextRequesterListener (new TextRequesterListener () {
      public void actionPerformed (final TextRequesterEvent event) {
        final String requester_text = event.getRequesterText ().trim ();
        if (requester_text.length () == 0) {
          new MessageDialog (frame, "name too short");
        } else {
          if (entry_group.getDefaultEntry ().setName (requester_text)) {
            // it worked
          } else {
            new MessageDialog (frame,
                               "could not set the name of the default entry");
          }
        }
      }
    });

    text_requester.show ();
  }

  /**
   *  Ask the user for a new default Entry then set it.
   **/
  private void setDefaultEntry () {
    final EntryChooser entry_chooser =
      new EntryChooser (entry_group, "choose a default entry:");

    final EntryChooserListener new_listener =
      new EntryChooserListener () {
        public void entryChoosen (EntryChooserEvent event) {
          entry_group.setDefaultEntry (event.getChoosenEntry ());
          entry_chooser.dispose ();
        }
      };

    entry_chooser.addEntryChooserListener (new_listener);

    entry_chooser.setVisible (true);
  }

  /**
   *  A vector containing the Entry objects that this EntryEdit object knows
   *  about.  This reference is obtained from owning_component.
   **/
  private EntryGroup entry_group;

  /**
   *  A vector containing one Checkbox or Label for each Entry in the
   *  EntryGroup object.
   **/
  private Vector entry_components = new Vector ();

  /**
   *  The Frame reference that was passed to the constructor.
   **/
  private Frame frame = null;
}

